package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.task.domain.JOBLock;
import cn.ibizlab.core.task.service.IJOBLockService;
import cn.ibizlab.core.task.filter.JOBLockSearchContext;

@Slf4j
@Api(tags = {"任务锁" })
@RestController("api-joblock")
@RequestMapping("")
public class JOBLockResource {

    @Autowired
    public IJOBLockService joblockService;

    @Autowired
    @Lazy
    public JOBLockMapping joblockMapping;

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Create-all')")
    @ApiOperation(value = "新建任务锁", tags = {"任务锁" },  notes = "新建任务锁")
	@RequestMapping(method = RequestMethod.POST, value = "/joblocks")
    @Transactional
    public ResponseEntity<JOBLockDTO> create(@RequestBody JOBLockDTO joblockdto) {
        JOBLock domain = joblockMapping.toDomain(joblockdto);
		joblockService.create(domain);
        JOBLockDTO dto = joblockMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Create-all')")
    @ApiOperation(value = "批量新建任务锁", tags = {"任务锁" },  notes = "批量新建任务锁")
	@RequestMapping(method = RequestMethod.POST, value = "/joblocks/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<JOBLockDTO> joblockdtos) {
        joblockService.createBatch(joblockMapping.toDomain(joblockdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Save-all')")
    @ApiOperation(value = "保存任务锁", tags = {"任务锁" },  notes = "保存任务锁")
	@RequestMapping(method = RequestMethod.POST, value = "/joblocks/save")
    public ResponseEntity<Boolean> save(@RequestBody JOBLockDTO joblockdto) {
        return ResponseEntity.status(HttpStatus.OK).body(joblockService.save(joblockMapping.toDomain(joblockdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Save-all')")
    @ApiOperation(value = "批量保存任务锁", tags = {"任务锁" },  notes = "批量保存任务锁")
	@RequestMapping(method = RequestMethod.POST, value = "/joblocks/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<JOBLockDTO> joblockdtos) {
        joblockService.saveBatch(joblockMapping.toDomain(joblockdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Get-all')")
    @ApiOperation(value = "获取任务锁", tags = {"任务锁" },  notes = "获取任务锁")
	@RequestMapping(method = RequestMethod.GET, value = "/joblocks/{joblock_id}")
    public ResponseEntity<JOBLockDTO> get(@PathVariable("joblock_id") BigInteger joblock_id) {
        JOBLock domain = joblockService.get(joblock_id);
        JOBLockDTO dto = joblockMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Update-all')")
    @ApiOperation(value = "更新任务锁", tags = {"任务锁" },  notes = "更新任务锁")
	@RequestMapping(method = RequestMethod.PUT, value = "/joblocks/{joblock_id}")
    @Transactional
    public ResponseEntity<JOBLockDTO> update(@PathVariable("joblock_id") BigInteger joblock_id, @RequestBody JOBLockDTO joblockdto) {
		JOBLock domain  = joblockMapping.toDomain(joblockdto);
        domain .setId(joblock_id);
		joblockService.update(domain );
		JOBLockDTO dto = joblockMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Update-all')")
    @ApiOperation(value = "批量更新任务锁", tags = {"任务锁" },  notes = "批量更新任务锁")
	@RequestMapping(method = RequestMethod.PUT, value = "/joblocks/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<JOBLockDTO> joblockdtos) {
        joblockService.updateBatch(joblockMapping.toDomain(joblockdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "检查任务锁", tags = {"任务锁" },  notes = "检查任务锁")
	@RequestMapping(method = RequestMethod.POST, value = "/joblocks/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody JOBLockDTO joblockdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(joblockService.checkKey(joblockMapping.toDomain(joblockdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Remove-all')")
    @ApiOperation(value = "删除任务锁", tags = {"任务锁" },  notes = "删除任务锁")
	@RequestMapping(method = RequestMethod.DELETE, value = "/joblocks/{joblock_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("joblock_id") BigInteger joblock_id) {
         return ResponseEntity.status(HttpStatus.OK).body(joblockService.remove(joblock_id));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Remove-all')")
    @ApiOperation(value = "批量删除任务锁", tags = {"任务锁" },  notes = "批量删除任务锁")
	@RequestMapping(method = RequestMethod.DELETE, value = "/joblocks/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<BigInteger> ids) {
        joblockService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "获取任务锁草稿", tags = {"任务锁" },  notes = "获取任务锁草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/joblocks/getdraft")
    public ResponseEntity<JOBLockDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(joblockMapping.toDto(joblockService.getDraft(new JOBLock())));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Default-all')")
	@ApiOperation(value = "获取DEFAULT", tags = {"任务锁" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/joblocks/fetchdefault")
	public ResponseEntity<List<JOBLockDTO>> fetchDefault(JOBLockSearchContext context) {
        Page<JOBLock> domains = joblockService.searchDefault(context) ;
        List<JOBLockDTO> list = joblockMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBLock-Default-all')")
	@ApiOperation(value = "查询DEFAULT", tags = {"任务锁" } ,notes = "查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/joblocks/searchdefault")
	public ResponseEntity<Page<JOBLockDTO>> searchDefault(@RequestBody JOBLockSearchContext context) {
        Page<JOBLock> domains = joblockService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(joblockMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

